#include "dataspace_server.h"
#include "ipc.h"
#include "pager.h"



/* Initialise the pager with a page mapper and the given flags controlling
   access to a file. */

Pager::Pager(PageMapper *mapper, flags_t flags)
: _start(0), _size(0), _mapper(mapper), _flags(flags)
{
    /* Some pagers may not be initialised with a mapper. */

    if (_mapper != NULL)
        _mapper->attach();
}

/* Close the pager. */

void Pager::close()
{
    if (_mapper != NULL)
        _mapper->detach();
}

/* Flush data to the file. */

long Pager::flush(offset_t populated_size, offset_t *size)
{
    _mapper->flush_all(_start, populated_size);

    *size = _mapper->get_data_size();
    return L4_EOK;
}

/* Resize the underlying file. */

long Pager::resize(offset_t *size)
{
    _mapper->set_data_size(*size);

    *size = _mapper->get_data_size();
    return L4_EOK;
}

/* Expose a region of the file. */

long Pager::mmap(offset_t position, offset_t length, offset_t *start_pos, offset_t *end_pos, offset_t *data_end)
{
    _start = trunc(position, PAGE_SIZE);
    _size = round(position + length, PAGE_SIZE) - _start;

    *start_pos = _start;
    *end_pos = _start + _size;
    *data_end = 0;

    return L4_EOK;
}

/* Map a flexpage corresponding to the dataspace 'offset' involving a 'hot_spot'
   (flexpage offset). */

long Pager::map(offset_t offset, l4_addr_t hot_spot, flags_t flags, l4_snd_fpage_t *region)
{
    offset_t file_offset = _start + offset;
    offset_t max_offset = _start + _size;
    Flexpage *flexpage = _mapper->get(file_offset, flags);

    /* Issue the flexpage via the IPC system. */

    long err = ipc_prepare_flexpage(flexpage, file_offset, max_offset, hot_spot,
                                    flags, region);

    if (!err)
      err = complete_Dataspace_map(*region);

    /* After the flexpage is issued, it is queued for future reuse. */

    _mapper->queue(flexpage);

    if (err)
      return err;

    return IPC_MESSAGE_SENT;
}

/* Return the total size of the data. */

offset_t Pager::get_data_size()
{
    return _mapper->get_data_size();
}

// vim: tabstop=4 expandtab shiftwidth=4
